@@ -3,11 +3,12 @@ use strict;
 use warnings;
 use autodie;
 
-use 5.008001;
+use 5.010000;
 
 use AlignDB::IntSpan;
 use Carp;
 use IO::Zlib;
+use IPC::Cmd;
 use List::MoreUtils;
 use Path::Tiny;
 use Tie::IxHash;
@@ -20,7 +21,7 @@ use vars qw(@ISA @EXPORT_OK %EXPORT_TAGS);
     all => [
         qw{
             read_sizes read_names read_replaces decode_header encode_header parse_block
-            parse_block_header parse_axt_block parse_maf_block revcom seq_length
+            parse_block_header parse_axt_block parse_maf_block revcom seq_length align_seqs
             },
     ],
 );
@@ -70,13 +71,13 @@ sub decode_header {
 
     # S288C.chrI(+):27070-29557|species=S288C
     my $head_qr = qr{
-        (?:(?P<name>[\w_]+)\.)?    
-        (?P<chr_name>[\w-]+)        
-        (?:\((?P<chr_strand>.+)\))? 
+        (?:(?P<name>[\w_]+)\.)?
+        (?P<chr_name>[\w-]+)
+        (?:\((?P<chr_strand>.+)\))?
         [\:]                        # spacer
-        (?P<chr_start>\d+)    
+        (?P<chr_start>\d+)
         [\_\-]                      # spacer
-        (?P<chr_end>\d+)        
+        (?P<chr_end>\d+)
     }xi;
 
     tie my %info, "Tie::IxHash";
@@ -106,9 +107,9 @@ sub decode_header {
     else {
         $name = $header;
         %info = (
-            name       => undef,
-            chr_name   => 'chrUn',
-            chr_strand => '+',
+            name       => $name,
+            chr_name   => undef,
+            chr_strand => undef,
             chr_start  => undef,
             chr_end    => undef,
         );
@@ -133,15 +134,17 @@ sub encode_header {
     my $only_essential = shift;
 
     my $header;
-    if ( defined $info->{name} ) {
-        $header .= $info->{name} . ".";
+    $header .= $info->{name};
+    if ( defined $info->{chr_name} ) {
+        $header .= "." . $info->{chr_name};
     }
-    $header .= $info->{chr_name};
     if ( defined $info->{chr_strand} ) {
         $header .= "(" . $info->{chr_strand} . ")";
     }
-    $header .= ":" . $info->{chr_start};
-    $header .= "-" . $info->{chr_end};
+    if ( defined $info->{chr_start} ) {
+        $header .= ":" . $info->{chr_start};
+        $header .= "-" . $info->{chr_end};
+    }
 
     # additional keys
     if ( !$only_essential ) {
@@ -304,4 +307,161 @@ sub seq_length {
     return length($seq) - $gaps;
 }
 
+sub indel_intspan {
+    my $seq = shift;
+
+    my $intspan = AlignDB::IntSpan->new;
+    my $length  = length($seq);
+
+    my $offset = 0;
+    my $start  = 0;
+    my $end    = 0;
+    for my $pos ( 1 .. $length ) {
+        my $base = substr( $seq, $pos - 1, 1 );
+        if ( $base eq '-' ) {
+            if ( $offset == 0 ) {
+                $start = $pos;
+            }
+            $offset++;
+        }
+        else {
+            if ( $offset != 0 ) {
+                $end = $pos - 1;
+                $intspan->add_pair( $start, $end );
+            }
+            $offset = 0;
+        }
+    }
+    if ( $offset != 0 ) {
+        $end = $length;
+        $intspan->add_pair( $start, $end );
+    }
+
+    return $intspan;
+}
+
+sub align_seqs {
+    my $seq_refs = shift;
+    my $aln_prog = shift;
+
+    # get executable
+    my $bin;
+
+    if ( !defined $aln_prog or $aln_prog =~ /clus/i ) {
+        $aln_prog = 'clustalw';
+        for my $e (qw{clustalw clustal-w clustalw2}) {
+            if ( IPC::Cmd::can_run($e) ) {
+                $bin = $e;
+                last;
+            }
+        }
+    }
+    elsif ( $aln_prog =~ /musc/i ) {
+        $aln_prog = 'muscle';
+        for my $e (qw{muscle}) {
+            if ( IPC::Cmd::can_run($e) ) {
+                $bin = $e;
+                last;
+            }
+        }
+    }
+    elsif ( $aln_prog =~ /maff/i ) {
+        $aln_prog = 'mafft';
+        for my $e (qw{mafft}) {
+            if ( IPC::Cmd::can_run($e) ) {
+                $bin = $e;
+                last;
+            }
+        }
+    }
+
+    if ( !defined $bin ) {
+        confess "Could not find the executable for $aln_prog\n";
+    }
+
+    # temp in and out
+    my $temp_in  = Path::Tiny->tempfile("seq_in_XXXXXXXX");
+    my $temp_out = Path::Tiny->tempfile("seq_out_XXXXXXXX");
+
+    # msa may change the order of sequences
+    my @indexes = 0 .. scalar( @{$seq_refs} - 1 );
+    {
+        my $fh = $temp_in->openw;
+        for my $i (@indexes) {
+            printf {$fh} ">seq_%d\n", $i;
+            printf {$fh} "%s\n",      $seq_refs->[$i];
+        }
+        close $fh;
+    }
+
+    my @args;
+    if ( $aln_prog eq "clustalw" ) {
+        push @args, "-align -type=dna -output=fasta -outorder=input -quiet";
+        push @args, "-infile=" . $temp_in->absolute->stringify;
+        push @args, "-outfile=" . $temp_out->absolute->stringify;
+    }
+    elsif ( $aln_prog eq "muscle" ) {
+        push @args, "-quiet";
+        push @args, "-in " . $temp_in->absolute->stringify;
+        push @args, "-out " . $temp_out->absolute->stringify;
+    }
+    elsif ( $aln_prog eq "mafft" ) {
+        push @args, "--quiet";
+        push @args, "--auto";
+        push @args, $temp_in->absolute->stringify;
+        push @args, "> " . $temp_out->absolute->stringify;
+    }
+
+    my $cmd_line = join " ", ( $bin, @args );
+    my $ok = IPC::Cmd::run( command => $cmd_line );
+
+    if ( !$ok ) {
+        Carp::confess("$aln_prog call failed\n");
+    }
+
+    my @aligned;
+    my $seq_of = read_fasta( $temp_out->absolute->stringify );
+    for my $i (@indexes) {
+        push @aligned, $seq_of->{ "seq_" . $i };
+    }
+
+    # delete .dnd files created by clustalw
+    #printf STDERR "%s\n", $temp_in->absolute->stringify;
+    if ( $aln_prog eq "clustalw" ) {
+        my $dnd = $temp_in->absolute->stringify . ".dnd";
+        path($dnd)->remove;
+    }
+
+    undef $temp_in;
+    undef $temp_out;
+
+    return \@aligned;
+}
+
+# read normal fasta files
+sub read_fasta {
+    my $filename = shift;
+
+    tie my %seq_of, "Tie::IxHash";
+    my @lines = path($filename)->lines;
+
+    my $cur_name;
+    for my $line (@lines) {
+        if ( $line =~ /^\>[\w:-]+/ ) {
+            $line =~ s/\>//;
+            chomp $line;
+            $cur_name = $line;
+            $seq_of{$cur_name} = '';
+        }
+        elsif ( $line =~ /^[\w-]+/ ) {
+            chomp $line;
+            $seq_of{$cur_name} .= $line;
+        }
+        else {    # Blank line, do nothing
+        }
+    }
+
+    return \%seq_of;
+}
+
 1;
